#!/bin/sh
# Script to check a jail status
# Args $1 = JAILNAME
######################################################################

# Source our functions
PROGDIR="/usr/local/share/warden"

# Source our variables
. ${PROGDIR}/scripts/backend/functions.sh

JAILNAME="$1"

if [ -z "${JAILNAME}" ]
then
  warden_error "You must specify a jail to check"
  exit 5
fi

if [ -z "${JDIR}" ]
then
  warden_error "JDIR is unset!!!!"
  exit 5
fi

JAILDIR="${JDIR}/${JAILNAME}"

if [ ! -d "${JAILDIR}" ]
then
  warden_error "No jail located at ${JAILDIR}"
  exit 5
fi

set_warden_metadir

IP4="$(warden_get_ipv4)"
if [ -n "${IP4}" ] ; then
  get_ip_and_netmask "${IP4}"
  IP4="${JIP}"
  MASK4="${JMASK}"
fi

IP6="$(warden_get_ipv6)"
if [ -n "${IP6}" ] ; then
  get_ip_and_netmask "${IP6}"
  IP6="${JIP}"
  MASK6="${JMASK}"
fi

# Display details about this jail now
#####################################################################
echo "Details for jail: ${JAILNAME}"
isDirZFS "${JAILDIR}" "1"
if [ $? -eq 0 ] ; then 
   tank="`getZFSDataset "${JAILDIR}"`"
   diskUsage="`df -m | grep -w "^${tank} " | awk '{print $3}'`"
else
   diskUsage="`du -c -x -m "${JAILDIR}" 2>/dev/null | grep total | tail -n 1 | awk '{print $1}'`"
fi

portList4=
activeCon4=
if [ -n "${IP4}" ] ; then
   sockstat | grep "${IP4}" | grep '*.*' | awk '{print $6}' | sed "s|${IP4}:||g" | sort -g | uniq >/tmp/.socklist4.$$
   while read line
   do
     if [ -z "$portList4" ] ; then
       portList4="${line}" 
     else
       portList4="${portList4},$line" 
     fi
   done < /tmp/.socklist4.$$
   rm /tmp/.socklist4.$$
   activeCon4=`sockstat | grep "${IP4}" | grep -v '*.*' | wc -l | awk '{print $1}'`
fi

portList6=
activeCon6=
if [ -n "${IP6}" ] ; then
   sockstat | grep "${IP6}" | grep '*.*' | awk '{print $6}' | sed "s|${IP6}:||g" | sort -g | uniq >/tmp/.socklist6.$$
   while read line
   do
     if [ -z "$portList6" ] ; then
       portList6="${line}" 
     else
       portList6="${portList6},$line" 
     fi
   done < /tmp/.socklist6.$$
   rm /tmp/.socklist6.$$
   activeCon6=`sockstat | grep "${IP6}" | grep -v '*.*' | wc -l | awk '{print $1}'`
fi

warden_print "Disk Usage: ${diskUsage}MB"

warden_print "Active IPv4 Ports: ${portList4}"
warden_print "Current IPv4 Connections: ${activeCon4}"

warden_print "Active IPv6 Ports: ${portList6}"
warden_print "Current IPv6 Connections: ${activeCon6}"
